%%
% LPVDelayExample.m
%
% M-file to analyze a LPV delayed system using IQCs and dissipation
% inequalities.  The delayed system is a Cutter taken from X. Zhang, 
% P. Tsiotras, and C. Knospe, Stability analysis of lpv timedelayed
% systems, International Journal of Control, vol. 75, no. 7, pp.
% 538558, 2002.
% The aim of the example is to obtain the time delay margins for different
% cutting stiffnesses k


clear all

% delay margin is obtained by bisection
tol = 1e-3; %tolerance for bisection

%stiffness values
kvec = [.26,.265,.27,.28,.3,.35,.4,.45];
for ik = 1:length(kvec)
    k = kvec(ik); %set stiffness
    
    %build plant
    % dot(x) = (A0 + rho*A1)*x + (B0 + rho*B1)*u
    % with rho =[ -1,1] as scheduling parameter
    A0 = [0, 0, 1, 0; ...
        0, 0, 0, 1;...
        -10-.171*k, 10, 0 , 0; ...
        5, -15, 0, -.25];
    
    B0 = [0, 0, 0, 0;...
        0, 0, 0, 0;...
        .171*k, 0, 0, 0;...
        0, 0, 0, 0];
    
    A1 = [0, 0, 0, 0;...
        0, 0, 0, 0;...
        .5*k, 0, 0, 0;...
        0, 0, 0, 0];
    
    B1 = [0, 0, 0, 0;...
        0, 0, 0, 0;...
        -.5*k, 0, 0, 0;...
        0, 0, 0, 0];
    
    L(:,:,1) = ss(A0+A1,B0+B1,eye(4),0);
    L(:,:,2) = ss(A0-A1,B0-B1,eye(4),0);
    
    
    % Perform loop-shift to represent reference to error transfer function
    % as Fu(Glin,Stau) where Stau(v) := Dtau(v)-v. Glin is [-T S; -T S] where
    % S/T are the sensitivity/complementary sensitivity transfer functions.
    % The Matlab expression below gives a minimal realization for Glin.
    A = A0 + A1;
    B = [B0(:,1)+B1(:,1) B0(:,1)+B1(:,1)];
    C = [1  0 0 0; 1 0 0 0];
    A = A + (B0(:,1)+B1(:,1))*[1  0 0 0];
    D = [0 1; 0 1];
    G(:,:,1) = ss(A,B,C,D);   
    
    A = A0 - A1;
    B = [B0(:,1)-B1(:,1) B0(:,1)-B1(:,1)];
    C = [1  0 0 0; 1 0 0 0];
    A = A + (B0(:,1)-B1(:,1))*[1  0 0 0];
    D = [0 1; 0 1];
    G(:,:,2) = ss(A,B,C,D);

    % IQC gridengine using intersection of IQC for circle at one and IQC 
    % for circle at halfpoint
    
    % Select the IQC multipliers to be used for analysis.
    % The IQC multipliers are constructed using the helper function DELAYIQC
    % based on the following types:
    %     -type==1: Circle IQC with center = -1 and radius = 1.
    %     -type==2: Circle IQC with center = 0 and radius = |phi|.
    %     -type==3: Circle IQC with center = |phi| and radius = |phi|.
    % The multipliers can be weighted by frequency since the delay is constant.
    
    IQCtype = [1; 3];
    IQCwt = [1; 1];
    
    % options for bisection algorithm
    rlb = 0; %lower bound
    rub = 100; %upper bound
    while rub-rlb>tol
        rtry = (rlb+rub)/2;
        
        % Specify IQCs
        PI(:,:,1) = delayiqc(IQCtype(1),rtry,IQCwt(1));
        PI(:,:,2) = delayiqc(IQCtype(2),rtry,IQCwt(2));
                       
        %run analysis
        [giqclmi,info] = lpviqcgain(G,PI);
        
        %new bounds for bisection
        if isempty(giqclmi)
            rub = rtry;
        else
            rlb = rtry;
        end
    end
    rmax1(ik) = rlb; %time delay margin for a given stiffness value
    
    
    % IQC gridengine
    % using intersection of IQC for circle at one and IQC for circle at origin
    
    % Select the IQC multipliers to be used for analysis.
    % The IQC multipliers are constructed using the helper function DELAYIQC
    % based on the following types:
    %     -type==1: Circle IQC with center = -1 and radius = 1.
    %     -type==2: Circle IQC with center = 0 and radius = |phi|.
    %     -type==3: Circle IQC with center = |phi| and radius = |phi|.
    % The multipliers can be weighted by frequency since the delay is constant.
    
    IQCtype = [1; 2];
    IQCwt = [1; 1];
    
    % options for bisection algorithm
    rlb = 0; %lower bound
    rub = 100; %upper bound
    while rub-rlb>tol
        rtry = (rlb+rub)/2;
        
        % Specify IQCs
        PI(:,:,1) = delayiqc(IQCtype(1),rtry,IQCwt(1));
        PI(:,:,2) = delayiqc(IQCtype(2),rtry,IQCwt(2));
                    
        %run analysis
        [giqclmi,info] = lpviqcgain(G,PI);

        %new bounds for bisection
        if isempty(giqclmi)
            rub = rtry;
        else
            rlb = rtry;
        end
    end
    rmax2(ik) = rlb; %time delay margin for a given stiffness value
    
    
    
    % Corollary 5.3 taken from X. Zhang, P. Tsiotras, and C. Knospe, 
    % Stability analysis of lpv timedelayed
    % systems, International Journal of Control, vol. 75, no. 7, pp.
    % 538558, 2002.
    
    % options for bisection algorithm
    rlb = 0; %lower bound
    rub = 100; %upper bound
    while rub-rlb>tol
        rtry = (rlb+rub)/2;
        
        % build lmis (see Zhang paper for details
        setlmis([])
        
        %define LMI variables
        [Q1,nQ1,xQ1] = lmivar(1,[4 1]);
        [Q2,nQ2,xQ2] = lmivar(1,[4 1]);
        [Q,nQ,xQ] = lmivar(1,[4 1]);
        
        [Z,nZ,xZ] = lmivar(1,[4 1]);
        [P,nP,xP] = lmivar(1,[4 1]);
        
        [R,nR,xR] = lmivar(2,[4 4]);
        [W,nW,xW] = lmivar(2,[4 4]);
        
        [Wt,nWt,xWt] = lmivar(3,xW');
        
        [a,b,c,d] = ssdata(L);
        
        cnt = 1;
        
        %define LMI conditions
        for ii = 1:2
            
            lmiterm([cnt 1 1 P],1,a(:,:,ii),'s');
            lmiterm([cnt 1 1 R],1,b(:,:,ii),'s');
            lmiterm([cnt 1 1 Q],1,1);
            lmiterm([-cnt 1 1 Q2],1,1);
            lmiterm([cnt 1 1 Z],a(:,:,ii)',a(:,:,ii));
            
            lmiterm([-cnt 1 2 Q2], 1,1);
            lmiterm([cnt 1 3 R],rtry, b(:,:,ii));
            lmiterm([cnt 1 3 W],1,1);
            
            lmiterm([cnt 1 4 P],1,b(:,:,ii));
            lmiterm([-cnt 1 4 R],1,b(:,:,ii));
            lmiterm([cnt 1 4 Q2],1,1);
            lmiterm([cnt 1 4 Z],a(:,:,ii)',b(:,:,ii));
            
            lmiterm([cnt 1 5 Q1],rtry*a(:,:,ii)',1);
            
            lmiterm([-cnt 2 2 Q2], 1,1);
            lmiterm([-cnt 2 2 Q1], 1,1);
            lmiterm([cnt 2 4 Q2],1,1);
            
            lmiterm([-cnt 3 3 Z],1,1);
            lmiterm([cnt 3 3 W],rtry,1,'s');
            
            lmiterm([-cnt 3 4 Wt],1,1);
            lmiterm([cnt 3 7 W],rtry,1);
            
            lmiterm([-cnt 4 4 Q],1,1);
            lmiterm([-cnt 4 4 Q2],1,1);
            lmiterm([cnt 4 4 Z],b(:,:,ii)',b(:,:,ii));
            
            lmiterm([cnt 4 6 Q2],rtry*a(:,:,ii)',1);
            
            lmiterm([-cnt 5 5 Q1],1,1);
            
            
            lmiterm([-cnt 6 6 Q2],1,1);
            lmiterm([cnt 6 7 0],zeros(4,4));
            
            lmiterm([-cnt 7 7 Z],1,1);
            
            cnt = cnt + 1;
        end
                
        % positivity condition on the LMI variables
        lmiterm([-cnt 1 1 Q],1,1);
        cnt = cnt +1;
        lmiterm([-cnt 1 1 Q1],1,1);
        cnt = cnt +1;
        lmiterm([-cnt 1 1 Q2],1,1);
        cnt = cnt +1;
        lmiterm([-cnt 1 1 Z],1,1);
        cnt = cnt +1;
        lmiterm([-cnt 1 1 P],1,1);
        
        %get lmis and run analysis
        lmisys = getlmis;
        opts = [0 0 0 0 1];
        [tmin,xfeas]=feasp(lmisys,opts);
        
        %new bounds for bisection        
        if tmin < 0
            rlb = rtry;
        else
            rub = rtry;
        end
        
    end
    rmax3(ik) = rlb; %time delay margin for a given stiffness value
    
end

%% plot results

figure(1)
hold on
plot(kvec,rmax1,'-','color',mycolor('maroon'),'MarkerSize',12)
plot(kvec,rmax2,':','color',mycolor('darkgray'),'MarkerSize',8)
plot(kvec,rmax3,'--','color',mycolor('coolblue'),'MarkerSize',12)

xlim([0.15 0.5])
ylim([0 2])
xlabel('cutting stiffness [-]')
ylabel('time delay margin [s]')
if exist('garyfyFigure','file')
    garyfyFigure;
end
box on
legend('Location','NorthEast')
legend('\Pi_1 and \Pi_3','\Pi_1 and \Pi_2','Cor. 5.3. Zhang2002');
% matlab2tikz('ex3results.tex')

